
/* suppath to the folder where this analysis is carried out */
global suppath "C:\Users\nurbat\Program" 


/* get programs for fitting FPM with multiple time-scales */
ssc install stmt, replace 

/*----------------------------------------------------------------------------*/	
/* Fit the FPM on the log hazard scale */		
/*----------------------------------------------------------------------------*/

/* bring in the prepared BC data */	
use "$suppath\supplementary_material\studycohort.dta", clear

/* create splines for age at surgery */
quietly rcsgen age, gen(age_rcs) df(5) center(40)
global age_knots `r(knots)'

/* create offset: time from surgery to relapse; 
  and make it negative because we want t2 to start at RM time */
quietly gen r_offset = cond(indic_rm==0, 0, -_t0)

/* FPM model on log hazard scale, where hazard rates are for the full disease history: 
t1 = time from surgery, 
t2 = time from relapse/metas = t1-r_offset 
*/
global bc_t1_knots -2.0828694 1.0016581 1.6371144 2.3020373
global bc_t2_knots 0.00136894 1.0143738 2.2847369 9.1540039

global bc_t1_bknots -2.0828694 2.3020373
global bc_t1_iknots 1.0016581 1.6371144  
global bc_t2_bknots 0.00136894 9.1540039
global bc_t2_iknots 1.0143738 2.2847369 

stmt indic_rm age_rcs1-age_rcs5 hormon, ///
	 time1(bknots($bc_t1_bknots) knots($bc_t1_iknots) knscale(log)) ///
	 time2(start(r_offset) bknots($bc_t2_bknots) knots($bc_t2_iknots) indicator(indic_rm) logtoff) nohr noorthog

quietly estimates save "$suppath/fpm.ster", replace

/*----------------------------------------------------------------------------*/	
/* predict hazard rates */		
/*----------------------------------------------------------------------------*/

/* get number of obs needed between 0 and 15yrs with step of 0.05 */
foreach j1 of numlist 0(0.05)15  {
	local obs1 = `obs1' + 1
}

/* create variables with time1 (tss), offset, time2(tsr) = time1 - offset */
forvalues c = 0(0.05)5  {
	qui clear 
	qui range time1 0 15 `obs1'
	qui replace time1 = round(time1, 0.01) 
	qui gen offset = round(`c', 0.01)
	qui gen time2 = round(time1 - `c', 0.01)
	qui tempfile temppred1`n1' 
	qui save `temppred1`n1'' 
	local datalist1 `datalist1' `temppred1`n1'' 
	local n1=`n1'+1 
} 
qui clear 
qui set obs 0 
qui append using `datalist1' 
qui order time1 time2

/* tv indicator for RM */
gen indic_rm = cond(time2<0, 0, 1)  
gen hormon = .
gen age = 50

/* create splines for reference age=50 at surgery */
qui rcsgen age, gen(age_rcs) knots($age_knots) center(40)

/* hazard rate for RM using math.formula */			
qui gen log_t=log(time1)
qui rcsgen log_t, gen (log_t_rcs) knots($bc_t1_knots) 		
qui rcsgen time2, gen(t2_rcs) knots($bc_t2_knots)
	
/* rm = 1 */
foreach hormon in 0 1 {
	qui gen fpm_mu_horm`hormon'_rm1=exp(_b[rcs:_cons] + _b[xb:indic_rm]*indic_rm + _b[xb:hormon]*`hormon' + ///
						 _b[xb:age_rcs1]*age_rcs1 + _b[xb:age_rcs2]*age_rcs2 + _b[xb:age_rcs3]*age_rcs3 + _b[xb:age_rcs4]*age_rcs4 + _b[xb:age_rcs5]*age_rcs5 + ///
						 _b[rcs:__t1_s1]*log_t_rcs1 + _b[rcs:__t1_s2]*log_t_rcs2 + _b[rcs:__t1_s3]*log_t_rcs3 + ///
						 indic_rm*(_b[rcs:__t2_s1]*t2_rcs1 + _b[rcs:__t2_s2]*t2_rcs2 + _b[rcs:__t2_s3]*t2_rcs3) ) * 1000
}

/* rm = 0 */
foreach hormon in 0 1 {						 
	qui gen fpm_mu_horm`hormon'_rm0=exp(_b[rcs:_cons] + _b[xb:indic_rm]*0 + _b[xb:hormon]*`hormon' + ///
						 _b[xb:age_rcs1]*age_rcs1 + _b[xb:age_rcs2]*age_rcs2 + _b[xb:age_rcs3]*age_rcs3 + _b[xb:age_rcs4]*age_rcs4 + _b[xb:age_rcs5]*age_rcs5 + ///
						 _b[rcs:__t1_s1]*log_t_rcs1 + _b[rcs:__t1_s2]*log_t_rcs2 + _b[rcs:__t1_s3]*log_t_rcs3 + ///
						 0*(_b[rcs:__t2_s1]*t2_rcs1 + _b[rcs:__t2_s2]*t2_rcs2 + _b[rcs:__t2_s3]*t2_rcs3) ) * 1000
}

/* predict log HR for RM vs non-RM */
qui gen _d = .
qui gen _t = .
cap drop t2_rcs*
qui rcsgen time2, knots(`e(bhknots_t2)') gen(t2_rcs)
qui predictnl log_hr = _b[xb:indic_rm]*1 + _b[rcs:__t2_s1]*t2_rcs1 + _b[rcs:__t2_s2]*t2_rcs2 + _b[rcs:__t2_s3]*t2_rcs3, ci(log_hr_lci log_hr_uci)
qui gen fpm_hr = exp(log_hr)
qui gen fpm_hr_lci = exp(log_hr_lci)
qui gen fpm_hr_uci = exp(log_hr_uci)
		   
keep time1 time2 offset indic_rm fpm_mu* fpm_hr*
rename indic_rm indic_for_rm1

quietly save "$suppath\supplementary_material\fpm_mu.dta", replace


/* plot hazards and HR */
/* along t1 = time since surgery */
twoway (line fpm_mu_horm1_rm0 time1, sort) ///
       (line fpm_mu_horm1_rm1 time1 if offset==0.5, sort) ///
       (line fpm_mu_horm1_rm1 time1 if offset==1, sort) ///
       (line fpm_mu_horm1_rm1 time1 if offset==2, sort) ///
       (line fpm_mu_horm1_rm1 time1 if offset==3, sort) ///
       (line fpm_mu_horm1_rm1 time1 if offset==4, sort) ///
       (line fpm_mu_horm1_rm1 time1 if offset==5, sort)  if time1<=10 ///
	   	, xsize(8.5) ysize(6) ///
		title(FPM: mortality rates) ///
	   	subtitle("on hormon trt, age = 50") ///
		ytitle(Mortality rate per 1000 years) ///
		xtitle(Time since surgery) ///
		xlabel(0[1]10) 	///		
		ylabel(0[100]700) ///
		legend(order(1 "Non Rel/Met" 2 "at 0.5y" ///
					3 "at 1y" 4 "at 2y" /// 
					5 "at 3y" 6 "at 4y" /// 
					7 "at 5y") symxsize(6.5) cols(3)) ///
		name(fpm_mu_tss, replace)
		
/* along t2 = time since relapse */
twoway (line fpm_mu_horm1_rm1 time2 if offset==0.5, sort) ///
	   (line fpm_mu_horm1_rm1 time2 if offset==1, sort) ///
	   (line fpm_mu_horm1_rm1 time2 if offset==2, sort) ///
	   (line fpm_mu_horm1_rm1 time2 if offset==3, sort) ///
	   (line fpm_mu_horm1_rm1 time2 if offset==4, sort) ///
	   (line fpm_mu_horm1_rm1 time2 if offset==5, sort) if 0<=time2 & time2<=7 ///
	   	, xsize(8.5) ysize(6) ///
		title(FPM: mortality rates) ///
	   	subtitle("on hormon trt, age = 50") ///
		ytitle(Mortality rate per 1000 years) ///
		xtitle(Time since relapse or metastasis) ///
		xlabel(0[1]7) ///		
		ylabel(0[100]700) ///
		legend(order(1 "at 0.5y" 2 "at 1y" 3 "at 2y" /// 
					4 "at 3y" 5 "at 4y" 6 "at 5y") symxsize(6.5) cols(3)) ///
		name(fpm_mu_tsr, replace)			   

/* plot hr for RM vs nonRM */		
twoway (rarea fpm_hr_lci fpm_hr_uci time2, sort pstyle(ci)) ///
	   	(line fpm_hr time2, sort) if time2>=0 & time2<=10 ///
		, xsize(8.5) ysize(6) ///
		title(FPM: mortality rate ratio) ///
		subtitle("RM vs non-RM") ///
		ytitle("Mortality rate ratio") ///
		xtitle(Time since relapse or metastasis) ///
		xlabel(0[1]10) ylabel(0[10]80) legend(off)	///		
		name(fpm_hr, replace)	


/*----------------------------------------------------------------------------*/	
/* predict survival proportions */		
/*----------------------------------------------------------------------------*/
		
qui keep time1 time2 offset indic_for_rm1 *mu* 
qui duplicates drop
sort time1 offset

/* survival before relapse/metastasis:
 1) for integ to work have to keep only relevant variables: t1, fpm hazard rates 
 2) use integ for estimating survival for no-rm patients 
 3) merge the survival estimates for no-rm patients with the original dataset */
preserve
keep time1 *_rm0
qui duplicates drop

foreach hormon in 0 1 {
	qui gen temp_mu_horm`hormon'_rm0 = fpm_mu_horm`hormon'_rm0/1000			
	qui integ temp_mu_horm`hormon'_rm0 time1, gen(fpm_cum_mu_horm`hormon'_rm0)
	qui gen fpm_s_horm`hormon'_rm0 = exp(-fpm_cum_mu_horm`hormon'_rm0)
	drop temp_*
} 

qui tempfile file_surv 
qui save `file_surv'

restore
qui merge m:1 time1 fpm_mu_horm1_rm0 fpm_mu_horm0_rm0 using `file_surv'
qui drop _merge

/* survival for patients who had relapse/metastasis:
 1) for integ to work have to keep only relevant variables: offset, t2, and fpm hazard rates 
 2) use integ for estimating survival for rm patients for each offset
 3) merge the survival estimates for rm patients with the original+surv_no_rm dataset */
qui tempfile orig 
qui save `orig'
 
forvalues c = 0(0.05)5  {
	
	qui use `orig', clear
	qui keep if offset==float(`c')
	
	foreach hormon in 0 1 {
			qui gen temp_mu_horm`hormon'_rm1 = fpm_mu_horm`hormon'_rm1/1000
			qui integ temp_mu_horm`hormon'_rm1 time2, gen(fpm_cum_mu_horm`hormon'_rm1)
			qui gen fpm_s_horm`hormon'_rm1 = exp(-fpm_cum_mu_horm`hormon'_rm1)
			drop temp_*
	}
	qui tempfile temppred2`n2' 
	qui save `temppred2`n2'' 
	local datalist2 `datalist2' `temppred2`n2'' 
	local n2=`n2'+1 
	clear
} 
qui clear 
qui set obs 0 
qui append using `datalist2' 

/* CONDITIONAL survival for patients who had relapse/metastasis (i.e. given patients survived up to RM):
 1) for integ to work have to keep only relevant variables: offset, t2, pois and fpm hazard rates 
 2) use integ for estimating survival for rm patients for each offset
 3) merge the survival estimates for rm patients with the original+surv_no_rm dataset */
qui tempfile orig2 
qui save `orig2'
 
forvalues c = 0(0.05)5  {
	
	qui use `orig2', clear
	qui keep if offset==float(`c') & time2>=0
	
	foreach hormon in 0 1 {
		qui gen temp_mu_horm`hormon'_rm1 = fpm_mu_horm`hormon'_rm1/1000
		qui integ temp_mu_horm`hormon'_rm1 time2, gen(fpm_cond_cum_mu_horm`hormon'_rm1)
		qui gen fpm_cond_s_horm`hormon'_rm1 = exp(-fpm_cond_cum_mu_horm`hormon'_rm1)
		drop temp_*
	}
	
	qui tempfile temppred`n2' 
	qui save `temppred`n2'' 
	local datalist2 `datalist2' `temppred`n2'' 
	local n2=`n2'+1 
	clear
} 
qui clear 
qui set obs 0 
qui append using `datalist2' 

qui merge 1:m time1 time2 offset indic_for_rm1 using `orig2'
qui drop _merge

quietly save "$suppath\supplementary_material\fpm_surv.dta", replace


/*----------------------------------------------------------------------------*/
/* Example of panel of 4 figures for displaying Survival */
/*----------------------------------------------------------------------------*/

/* over tss */
twoway (line fpm_s_horm1_rm1 time1 if offset==1, sort lcolor(forest_green)) ///
       (line fpm_s_horm1_rm1 time1 if offset==2, sort lcolor(dkorange)) ///
       (line fpm_s_horm1_rm1 time1 if offset==3, sort lcolor(emidblue)) ///
       (line fpm_s_horm1_rm1 time1 if offset==4, sort lcolor(erose)) ///
       (line fpm_s_horm1_rm1 time1 if offset==5, sort lcolor(lavender)) if time1<=10 ///
	   	, xsize(8.5) ysize(6) ///
		xlabel(0[1]10) ///		
		ylabel(0[0.2]1) ///
		ytitle(Proportion) ///
		xtitle("Time since surgery (time1)", size(small)) ///
		legend(subtitle("Time of RM", size(small)) size(small) ///
				order(1 "1" 2 "2" /// 
					3 "3" 4 "4" 5 "5") ///
					symxsize(6.5) nobox ///
           region(lstyle(none)) cols(1) ring(0) pos(1) bmargin(zero) rowgap(0) colgap(0) ) ///
		name(s_fpm_11, replace) 		

/* over tsr */
twoway (line fpm_s_horm1_rm1 time2 if offset==1, sort lcolor(forest_green)) ///
       (line fpm_s_horm1_rm1 time2 if offset==2, sort lcolor(dkorange)) ///
       (line fpm_s_horm1_rm1 time2 if offset==3, sort lcolor(emidblue)) ///
       (line fpm_s_horm1_rm1 time2 if offset==4, sort lcolor(erose)) ///
       (line fpm_s_horm1_rm1 time2 if offset==5, sort lcolor(lavender)) if 0<=time2 & time2<=5 ///
	   	, xsize(8.5) ysize(6) ///
		xlabel(0[1]5) ///		
		ylabel(0[0.2]1) ///
		ytitle(Proportion) ///
		xtitle(Time since RM (time2), size(small)) ///
		legend(subtitle("Time of RM", size(small)) size(small) ///
				order(1 "1" 2 "2" /// 
					3 "3" 4 "4" 5 "5") ///
					symxsize(6.5) nobox ///
           region(lstyle(none)) cols(1) ring(0) pos(1) bmargin(zero) rowgap(0) colgap(0) ) ///
		name(s_fpm_21, replace) 

/* over tss for different tsr */
twoway	(line fpm_s_horm1_rm1 time1 if time2==float(1), sort lcolor(navy)) ///
		(line fpm_s_horm1_rm1 time1 if time2==float(3), sort lcolor(maroon)) ///
		(line fpm_s_horm1_rm1 time1 if time2==float(5), sort lcolor(dkorange)) ///
	   	, xsize(8.5) ysize(6) ///
		xlabel(0[1]10) ///		
		ylabel(0[0.2]1) ///
		ytitle(Proportion) ///
		xtitle("Time since surgery (time1)", size(small)) ///
		legend(subtitle("Time since RM", size(small)) size(small) ///
				order(1 "1" 2 "3" 3 "5") ///
					symxsize(6.5) nobox ///
           region(lstyle(none)) cols(1) ring(0) pos(7) bmargin(zero) rowgap(0) colgap(0) ) ///
		name(s_fpm_12, replace) 
		
		
/* over tsr for different tss */
twoway (line fpm_s_horm1_rm1 time2 if time1==float(1), sort lcolor(navy) ) ///
		(line fpm_s_horm1_rm1 time2 if time1==float(3), sort lcolor(maroon)) ///
		(line fpm_s_horm1_rm1 time2 if time1==float(5), sort lcolor(dkorange)) if 0<=time2 & time2<=5  ///		
	   	, xsize(8.5) ysize(6) ///
		xlabel(0[1]5) ///		
		ylabel(0[0.2]1) ///
		ytitle(Proportion) ///
		xtitle(Time since RM (time2), size(small)) ///
		legend(subtitle("Time since" "surgery", size(small)) size(small) ///
				order(1 "1" 2 "3" 3 "5") ///
					symxsize(6.5) nobox ///
           region(lstyle(none)) cols(1) ring(0) pos(7) bmargin(zero) rowgap(0) colgap(0)) ///
		name(s_fpm_22, replace) 
		
graph combine s_fpm_11 s_fpm_21 s_fpm_12 s_fpm_22, ///
		col(2) imargin(0 0 1 1) title("Survival for RM on hormon trt, age = 50", size(small)) ///
		name(comb_s_fpm, replace)
