

/*----------------------------------------------------------------------------*/	
/* create main frame for saving all surv.predictions */
/*----------------------------------------------------------------------------*/
frame create main_frame
frame change main_frame

rename indic_for_rm1 indic_rm
qui keep time1 time2 offset indic_rm fpm_s_horm1_rm1 fpm_cond_s_horm1_rm1 
qui gen tempID = _n

frame copy main_frame init_main_frame

/*----------------------------------------------------------------------------*/	
/* many bootstraps */
/*----------------------------------------------------------------------------*/	

timer on 12

set seed 20201107
forvalues b = 1(1)1000 {

/*----------------------------------------------------------------------------*/	
/* run 1 bootstrap */
/*----------------------------------------------------------------------------*/	

frame create boot_frame
frame change boot_frame
qui use "$suppath\supplementary_material\studycohort.dta", clear

bsample, cluster(pid) idcluster(new_pid)   

quietly rcsgen age, gen(age_rcs) df(5) center(40)
local boot_age_knots `r(knots)'

quietly gen r_offset = cond(indic_rm==0, 0, -_t0)

/* FPM model on log hazard scale, where hazard rates are for the full disease history: 
t1 = time from surgery, 
t2 = time from relapse/metas = t1-r_offset 
*/
quietly stmt indic_rm age_rcs1-age_rcs5 hormon, ///
	 time1(df(3)) time2(start(r_offset) df(3) indicator(indic_rm) logtoff) nohr noorthog

/*----------------------------------------------------------------------------*/	
/* predict hazard rates */		
/*----------------------------------------------------------------------------*/

/* get number of obs needed between 0 and 15yrs with step of 0.05 */
foreach j1 of numlist 0(0.05)15  {
	local obs`b' = `obs`b'' + 1
}

/* create variables with time1 (tss), offset, time2(tsr) = time1 - offset */
forvalues c = 0(0.05)5  {
	qui clear 
	qui range time1 0 15 `obs`b''
	qui replace time1 = round(time1, 0.01)  // have to use round due to Stata rounding issues
	qui gen offset = round(`c', 0.01)
	qui gen time2 = round(time1 - `c', 0.01)
	qui tempfile temppred`b'1`n1' 
	qui save `temppred`b'1`n1'' 
	local datalist1`b' `datalist1`b'' `temppred`b'1`n1'' 
	local n=`n1'+1 
} 
qui clear 
qui set obs 0 
qui append using `datalist1`b'' 
qui order time1 time2

/* tv indicator for RM */
qui gen indic_rm = cond(time2<0, 0, 1)  
qui gen hormon = .
qui gen age = 50

/* create splines for reference age=50 at surgery */
qui rcsgen age, gen(age_rcs) knots(`boot_age_knots') center(40)

/* hazard rate for RM using math.formula */			
qui gen log_t=log(time1)
qui rcsgen log_t, gen (log_t_rcs) knots(`e(bhknots_t1)') 		
qui rcsgen time2, gen(t2_rcs) knots(`e(bhknots_t2)')
	
/* rm = 1, hormon = 1 */
qui gen fpm_mu_horm1_rm1=exp(_b[rcs:_cons] + _b[xb:indic_rm]*indic_rm + _b[xb:hormon]*1 + ///
					 _b[xb:age_rcs1]*age_rcs1 + _b[xb:age_rcs2]*age_rcs2 + _b[xb:age_rcs3]*age_rcs3 + _b[xb:age_rcs4]*age_rcs4 + _b[xb:age_rcs5]*age_rcs5 + ///
					 _b[rcs:__t1_s1]*log_t_rcs1 + _b[rcs:__t1_s2]*log_t_rcs2 + _b[rcs:__t1_s3]*log_t_rcs3 + ///
					 indic_rm*(_b[rcs:__t2_s1]*t2_rcs1 + _b[rcs:__t2_s2]*t2_rcs2 + _b[rcs:__t2_s3]*t2_rcs3) ) * 1000
		 
/*----------------------------------------------------------------------------*/	
/* predict survival proportions */		
/*----------------------------------------------------------------------------*/
sort time1 offset

qui tempfile orig`b' 
qui save `orig`b''
 
forvalues c = 0(0.05)5  {
	
	qui use `orig`b'', clear
	qui keep if offset==float(`c')
	
	qui gen temp_mu_horm1_rm1 = fpm_mu_horm1_rm1/1000
	qui integ temp_mu_horm1_rm1 time2, gen(fpm_cum_mu_horm1_rm1)
	qui gen boot_surv = exp(-fpm_cum_mu_horm1_rm1)
	drop temp_*
		
	qui tempfile temppred`b'2`n2' 
	qui save `temppred`b'2`n2'' 
	local datalist2`b' `datalist2`b'' `temppred`b'2`n2'' 
	local n=`n2'+1 
	clear
} 
qui clear 
qui set obs 0 
qui append using `datalist2`b'' 
qui keep time1 time2 offset indic_rm fpm_mu_horm1_rm1 boot_surv
qui gen boot_n = `b'

/*CONDITIONAL survival*/
qui tempfile orig2`b' 
qui save `orig2`b''

forvalues c = 0(0.05)5  {
	
	qui use `orig2`b'', clear
	qui keep if offset==float(`c') & time2>=0
	
	qui gen temp_mu_horm1_rm1 = fpm_mu_horm1_rm1/1000
	qui integ temp_mu_horm1_rm1 time2, gen(fpm_cond_cum_mu_horm1_rm1)
	qui gen boot_cond_surv = exp(-fpm_cond_cum_mu_horm1_rm1)
	drop temp_*
		
	qui tempfile temppred3`b'`n3' 
	qui save `temppred3`b'`n3'' 
	local datalist3`b' `datalist3`b'' `temppred3`b'`n3'' 
	local n=`n3'+1 
	clear
} 
qui clear 
qui set obs 0 
qui append using `datalist3`b'' 
qui keep time1 time2 offset indic_rm boot_cond_surv
qui merge 1:m time1 time2 offset indic_rm using `orig2`b''
qui drop _merge

qui keep time1 time2 offset indic_rm boot_n boot_surv boot_cond_surv

qui frlink 1:1 time1 time2 offset, frame(init_main_frame)
qui frget fpm_s_horm1_rm1 fpm_cond_s_horm1_rm1 tempID, from(init_main_frame)
drop init_main_frame

tempfile boot_file
qui save `boot_file', replace
frame change main_frame

if (`b'==1) {
	qui joinby tempID time1 time2 offset indic_rm fpm_s_horm1_rm1 fpm_cond_s_horm1_rm1 using `boot_file'
}
if (`b'!=1) {
	qui append using `boot_file'
}

frame drop boot_frame

disp as result "Finished for boot sample `b'"

} // end of many bootstraps
timer off 12
timer list 12 //  2997.4410 ~ 50 mins

sort tempID time1 offset boot_n
qui save "$suppath\supplementary_material\fpm_surv_boot_samples.dta", replace  // 30,401,000 obs

/*----------------------------------------------------------------------------*/
/* Bias-corrected percentile CI */
/*----------------------------------------------------------------------------*/

/* select specific time-points of interest */
qui keep if inlist(time2, 0, 1, 3, 5) & inlist(time1, 4, 5, 8)
rename tempID old_tempID
egen tempID = group(old_tempID)

/* Bias-corrected percentile CI
1) Proportion of times that the boot estimate is lower than the observed estimate 
	a) If boot value is < observed value, code 1, otherwise 0 
	b) Mean is the proportion of times boot value <  obs value
2) Use proportion to get Z score, then use that to calculate the new bias-correct 
   Z score to look up the pth quantile of the bootstrap distribution */
gen boot_compare = cond(boot_surv<fpm_s_horm1_rm1, 1, 0)
egen boot_mean_low = mean(boot_compare), by(tempID) 
gen z0 = invnormal(boot_mean_low)
gen p_l = normal(2*z0 - invnormal(0.975))
gen p_u = normal(2*z0 + invnormal(0.975))

/* for conditional survival */
gen boot_compare_cond = cond(boot_cond_surv<fpm_cond_s_horm1_rm1, 1, 0)
egen boot_mean_low_cond = mean(boot_compare_cond), by(tempID) 
gen z0_cond = invnormal(boot_mean_low_cond)
gen p_l_cond = normal(2*z0_cond - invnormal(0.975))
gen p_u_cond = normal(2*z0_cond + invnormal(0.975))

cap drop boot_ci_*
gen boot_ci_l=.
gen boot_ci_u=.
gen boot_ci_l_cond=.
gen boot_ci_u_cond=.
tab tempID
forvalues i= 1(1)`r(r)' {
	summ p_l if tempID==`i', meanonly
	local plvalue=r(mean)*100
	cap egen boot_ci_l_`i' = pctile(boot_surv) if tempID==`i', p(`plvalue')  
	cap replace boot_ci_l=boot_ci_l_`i' if tempID==`i'
	cap drop boot_ci_l_`i'
	
	summ p_u if tempID==`i', meanonly
	local puvalue=r(mean)*100
	cap egen boot_ci_u_`i' = pctile(boot_surv) if tempID==`i', p(`puvalue')  
	cap replace boot_ci_u=boot_ci_u_`i' if tempID==`i'
	cap drop boot_ci_u_`i'	
	
	summ p_l_cond if tempID==`i', meanonly
	local plvalue_cond=r(mean)*100
	cap egen boot_ci_l_cond_`i' = pctile(boot_cond_surv) if tempID==`i', p(`plvalue_cond')  
	cap replace boot_ci_l_cond=boot_ci_l_cond_`i' if tempID==`i'
	cap drop boot_ci_l_cond_`i'
	
	summ p_u_cond if tempID==`i', meanonly
	local puvalue_cond=r(mean)*100
	cap egen boot_ci_u_cond_`i' = pctile(boot_cond_surv) if tempID==`i', p(`puvalue_cond')  
	cap replace boot_ci_u_cond=boot_ci_u_cond_`i' if tempID==`i'
	cap drop boot_ci_u_cond_`i'
}

keep time1 time2 offset indic_rm fpm_s_horm1_rm1 fpm_cond_s_horm1_rm1 boot_ci*
duplicates drop
save "$suppath\supplementary_material\fpm_surv_boot_CI.dta", replace 
